import { Box } from '../../classes/box.js';
import { Scene } from '../../classes/scene.js';
import { Expert } from '../../classes/expert.js';
import { Bench } from '../../classes/bench.js';
import { delay } from '../../util.js';
import { Forum } from '../../classes/forum.js';
import { PostContent } from '../../classes/post-content.js';
import params from '../../params.js';

const DEFAULT_DELAY_MS = 1;
const POOL_DURATION_MS = 50;

let rootElement;
let rootBox;
let scene;
let forum;
let bench;
let experts;
let posts;

async function newExpert() {
  const index = experts.length;
  const name = `Expert${index + 1}`;
  const expert = await new Expert(name, scene).initialize();
  experts.push(expert);
  return expert;
}

async function addPost(author, fee, citations = []) {
  const postIndex = posts.length;
  const title = `posts[${postIndex}]`;
  await scene.startSection();

  const postContent = new PostContent({}).setTitle(title);
  for (const { postId, weight } of citations) {
    postContent.addCitation(postId, weight);
  }

  const { pool, postId } = await author.submitPostWithFee(
    bench,
    forum,
    postContent,
    {
      fee,
      duration: POOL_DURATION_MS,
      tokenLossRatio: 1,
    },
  );
  posts.push(postId);
  await delay(POOL_DURATION_MS);
  await pool.evaluateWinningConditions();
  await scene.endSection();
  await delay(DEFAULT_DELAY_MS);
  return postId;
}

async function setup() {
  rootElement = document.getElementById('scene');
  rootBox = new Box('rootBox', rootElement).flex();

  scene = (window.scene = new Scene('Forum test', rootBox));
  scene.withSequenceDiagram();

  scene.addDisplayValue('c3. stakeForAuthor').set(params.stakeForAuthor);
  scene.addDisplayValue('q2. revaluationLimit').set(params.revaluationLimit);
  scene
    .addDisplayValue('q3. referenceChainLimit')
    .set(params.referenceChainLimit);
  scene.addDisplayValue('q4. leachingValue').set(params.leachingValue);
  scene.addDisplayValue('&nbsp;');

  forum = (window.forum = new Forum('Forum', scene));
  bench = (window.bench = new Bench(forum, 'Bench', scene));
  experts = (window.experts = []);
  posts = (window.posts = []);

  forum.posts.withFlowchart();

  scene.withTable();

  await newExpert();
  // await newExpert();
  // await newExpert();

  // bench.addValue('total rep', () => bench.reputation.getTotal());
  forum.addValue('total value', () => forum.getTotalValue());

// for (const expert of experts) {
// expert.addValue('rep', () => bench.reputation.valueOwnedBy(expert.reputationPublicKey));
// }
}

describe('Negative citation of a negative citation', async () => {
  before(async () => {
    await setup();
    await addPost(experts[0], 10);
    await addPost(experts[0], 10, [{ postId: posts[0], weight: -1 }]);
    await addPost(experts[0], 10, [{ postId: posts[1], weight: -1 }]);
  });
  it('Should restore original post to its initial value', () => {
    forum.getPost(posts[0]).value.should.equal(10);
  });
});

// await addPost(experts[0], 10);
// await addPost(experts[0], 10, [{ postId: posts[3], weight: -1 }]);
// await addPost(experts[0], 10, [{ postId: posts[4], weight: -1 }]);

// await addPost(expert3, 'Post 4', 100, [{ postId: postId2, weight: -1 }]);
// await addPost(expert1, 'Post 5', 100, [{ postId: postId3, weight: -1 }]);
